<?php


/**
 * Form builder function
 */
function menu_multicrud_form(&$form_state, $menu) {
  drupal_set_title("Multicrud menu <i>$menu[title]</i>");
  
  $form = array();
  $form['description'] = array(
    '#type' => 'markup',
    '#value' => '<p>' . t('Mass edit menu items') . '</p>',
  );
  
  $form['tree'] = _menu_multicrud_items_element($menu);
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Save',
    '#attributes' => array('tabindex' => 1),
  );
  
  $form['#menu'] = $menu;
  
  // generic submit handler provided by multicrud core.
  $form['#submit'] = array('multicrud_submit_all');
  
  return $form;
}



/**
 * The actual multicrud element
 */
function _menu_multicrud_items_element($menu) {
  
  $element = array(
    '#type' => 'multicrud',
    
    // enable tabledrag
    '#multicrud_weights' => TRUE,
    
    // enable hierarchical tabledrag
    '#multicrud_parents' => TRUE,
    
    // form elements for each row
    '#multicrud_elements' => _menu_multicrud_row_elements(),
    
    // value handler to load and save
    '#multicrud_valueHandler' => _menu_multicrud_value_handler($menu),
    
    // multicrud options
    '#multicrud' => array(
      'parent_key' => 'plid',
      'weight_key' => 'weight',
    ),
  );
  
  return $element;
}



function _menu_multicrud_row_elements() {

  $elements = array();
  $elements['name'] = array(
    '#input' => TRUE,
    // '#value' => t('Menu item'),
  );
  $elements['link_title'] = array(
    '#title' => 'Title',
    '#type' => 'textfield',
    '#size' => 25,
  );
  $elements['link_path'] = array(
    '#title' => 'Path',
    '#type' => 'textfield',
    '#size' => 25,
  );
  $elements['description'] = array(
    '#title' => 'Description',
    '#type' => 'textarea',
    '#cols' => 8,
    '#rows' => 1,
    '#resizable' => FALSE,
  );
  $elements['hidden'] = array(
    '#title' => 'Vis.',
    '#type' => 'checkbox',
  );
  $elements['expanded'] = array(
    '#title' => 'Exp.',
    '#type' => 'checkbox',
  );
  if ($language_options) {
    $elements['language'] = array(
      '#title' => 'Language',
      '#type' => 'select',
      '#options' => $language_options,
    );
  }
  
  return $elements;
}


/**
 * Factory function for the value handler
 */
function _menu_multicrud_value_handler($menu) {
  return new menu_multicrud_MulticrudValueHandler($menu);
}



/**
 * The value handler is responsible for loading, saving and validation of items.
 */
class menu_multicrud_MulticrudValueHandler extends multicrud_ValueHandler {

  protected $_menu;
  protected $_originalItems;
  protected $_placeholders = array();
  protected $_saveErrors = array();


  /**
   * @param $type
   *   the node type
   * @param $langSpecificFields
   *   names of fields that are not shared across languages
   */
  function __construct($menu) {
    $this->_menu = $menu;
    $this->_originalItems = _menu_multicrud_default_value($menu);
    
    
    // placeholders to change the link path
    if (function_exists('menu_editor_get_placeholders')) {
      $this->_placeholders = menu_editor_get_placeholders();
    }
    
    foreach ($this->_originalItems as $mlid => $item) {
      $this->_originalItems[$mlid]['original_link_path'] = $item['link_path'];
      foreach ($this->_placeholders as $code => $path) {
        if (str_replace('%mlid', $mlid, $path) == $item['link_path']) {
          $this->_originalItems[$mlid]['link_path'] = $code;
        }
      }
    }
  }
  
  
  function validateItem($item, $meta, $error_handler) {
    // TODO: Implement item validation
    if (!is_string($item['link_title']) || empty($item['link_title'])) {
      return FALSE;
    }
    if (!is_string($item['link_path']) || empty($item['link_path'])) {
      return FALSE;
    }
    return FALSE;
  }
  
  
  function getDefaultValue() {
    $transformed_items = array();
    foreach ($this->_originalItems as $mlid => $item) {
      $transformed_items[$mlid] = $this->_transformItem($item);
    }
    return $transformed_items;
  }


  /**
   * Called from multicrud for to-be-deleted items.
   */
  function deleteItem($mlid) {
    if (!isset($this->_originalItems[$mlid])) {
      // no item to delete.
      return;
    }
    
    menu_link_delete($mlid);
    // patch from http://drupal.org/node/786230#comment-2911158
    if (module_exists('i18nmenu')) {
      _i18nmenu_delete_item($v['mlid']);
    }
    return TRUE;
  }


  /**
   * Called from multicrud for to-be-updated items.
   */
  function updateItem($mlid, $item, $meta) {
    if (!$this->_validateItem($item)) {
      return FALSE;
    }
    if (!isset($this->_originalItems[$mlid])) {
      return;
    }
    $untransformed = $this->_untransformItem($item);
    $item = $this->_originalItems[$mlid][$field];
    $need_update = FALSE;
    foreach ($untransformed as $field => $value) {
      if ($value != $item[$field]) {
        $item[$field] = $value;
        $need_update = TRUE;
      }
    }
    if ($need_update) {
      $item['mlid'] = $mlid;
      $this->_saveItem($item);
    }
  }


  /**
   * Called from multicrud for to-be-created items.
   */
  function insertItem($item, $meta) {
    if (!$this->_validateItem($item)) {
      return FALSE;
    }
    
    $item = $this->_untransformItem($item);
    
    return $this->_saveItem($item);
  }

  protected function _saveItem($item) {
    $item['customized'] = 1;
    
    $link_path = &$item['link_path'];
    
    $link_path_placeholder = NULL;
    
    // placeholders
    if (isset($this->_placeholders[$link_path])) {
      if (is_numeric($item['mlid'])) {
        $link_path = str_replace('%mlid', $item['mlid'], $this->_placeholders[$link_path]);
      }
      else {
        $link_path_placeholder = $this->_placeholders[$link_path];
        // use a dummy link path,
        // until we know the correct mlid.
        $link_path = '<front>';
      }
    }
    
    // clean the link path
    if (isset($link_path)) {
      $link_path = drupal_get_normal_path($link_path);
      if (!menu_path_is_external($link_path)) {
        $parsed_link = parse_url($link_path);
        
        if (isset($parsed_link['query'])) {
          $item['options']['query'] = $parsed_link['query'];
        }
        else {
          unset($item['options']['query']);
        }
        
        if (isset($parsed_link['fragment'])) {
          $item['options']['fragment'] = $parsed_link['fragment'];
        }
        else {
          unset($item['options']['fragment']);
        }
        
        if ($link_path != $parsed_link['path']) {
          $link_path = $parsed_link['path'];
        }
      }
      if (!trim($link_path) || !menu_valid_path($item)) {
        // invalid link path, discard this item
        return FALSE;
      }
    }
    
    $mlid = menu_link_save($item);
    
    if (is_numeric($mlid)) {
      if (isset($link_path_placeholder)) {
        // overwrite the dummy link path
        $link_path = str_replace('%mlid', $item['mlid'], $link_path_placeholder);
        
        menu_link_save($item);
      }
      
      if (module_exists('i18nmenu')) {
        // Ensure we have a menu item to work with.
        // patch from http://drupal.org/node/786230#comment-2911038
        $item['parent'] = $item['menu_name'] . ':' . $item['plid'];
        _i18nmenu_update_item($item);
      }
    }
    else {
      return FALSE;
    }
  }

  protected function _validateItem($item) {
    if (!is_string($item['link_title']) || !strlen($item['link_title'])) {
      return FALSE;
    }
    if (!is_string($item['link_path']) || !strlen($item['link_path'])) {
      return FALSE;
    }
    return TRUE;
  }

  protected function _transformItem($item) {
    return _menu_multicrud_extract_item($item);
  }

  /**
   * Turn form values into a saveable menu item.
   */
  protected function _untransformItem($transformed) {
    $item = array();
    $item['hidden'] = !$transformed['hidden'];
    $fields = array('expanded', 'weight', 'plid', 'link_title', 'link_path');
    foreach ($fields as $field) {
      $item[$field] = $transformed[$field];
    }
    if (!isset($transformed['plid'])) {
      $item['plid'] = 0;
    }
    if (isset($transformed['language'])) {
      $item['options']['langcode'] = $transformed['language'];
    }
    if (isset($transformed['description'])) {
      $item['options']['attributes']['title'] = $transformed['description'];
    }
    $item['menu_name'] = $this->_menu['menu_name'];
    return $item;
  }
}




function _menu_multicrud_default_value($menu) {
  
  $q = db_query(
    $sql = "
      SELECT
        m.load_functions, m.to_arg_functions, m.access_callback, m.access_arguments,
        m.page_callback, m.page_arguments, m.title, m.title_callback, m.title_arguments,
        m.type, m.description, ml.*
      FROM
        {menu_links} ml
        LEFT JOIN {menu_router} m ON m.path = ml.router_path
      WHERE
        ml.menu_name = '%s'
      ORDER BY
        p1 ASC, p2 ASC, p3 ASC, p4 ASC, p5 ASC, p6 ASC, p7 ASC, p8 ASC, p9 ASC
    ",
    $menu['menu_name']
  );
  
  $tree = menu_tree_data($q);
  $node_links = array();
  menu_tree_collect_node_links($tree, $node_links);
  // We indicate that a menu administrator is running the menu access check. 
  $GLOBALS['menu_admin'] = TRUE;
  menu_tree_check_access($tree, $node_links);
  $GLOBALS['menu_admin'] = FALSE;
  
  $flat = array();
  _menu_multicrud_extract_tree($flat, $tree);
  
  return $flat;
}


function _menu_multicrud_extract_tree(&$flat, $tree) {
  foreach ($tree as $data) {
    // Don't show callbacks; these have $item['hidden'] < 0.
    if (!isset($data['link']) || $data['link']['hidden'] < 0) {
      continue;
    }
    
    $flat[$data['link']['mlid']] = $data['link'];
    
    // process child elements
    if ($data['below']) {
      _menu_multicrud_extract_tree($flat, $data['below']);
    }
  }
}


function _menu_multicrud_extract_item($item) {

  $path = $item['link_path'];
  if (is_string($item['options']['query'])) {
    $path .= '?'. $item['options']['query'];
  }
  if (is_string($item['options']['fragment'])) {
    $path .= '#' . $item['options']['fragment'];
  }
  
  return array(
    'name' => l($item['link_title'], $item['original_link_path']),
    'link_title' => $item['link_title'],
    'link_path' => $path,
    'description' => isset($item['options']['attributes']['title']) ? $item['options']['attributes']['title'] : '',
    'hidden'   => !$item['hidden'],
    'expanded' => $item['expanded'],
    'weight'   => $item['weight'],
    'plid'     => isset($item['plid']) ? $item['plid'] : 0,
    'language' => '' . @$item['options']['langcode'],
    'deleteable' => ($item['module'] == 'menu' || $item['updated'] == 1),
  );
}



