<?php


function multicrud_process($multi_element, $edit, &$form_state, $form) {

  $multi_element['#multicrud'] += array(
    'type_key' => 'type',
    'weight_key' => 'weight',
    'parent_key' => 'parent',
  );
  
  if (!$multi_element['#multicrud_weights']) {
    unset($multi_element['#multicrud']['weight_key']);
  }
  
  if (!$multi_element['#multicrud_parents']) {
    unset($multi_element['#multicrud']['parent_key']);
  }

  $multi_element['#multicrudElement'] = new multicrud_MulticrudElement($multi_element, $form['form_id']['#value'], $form_state);
  $form_state['#multicrudElements'][] = $multi_element['#multicrudElement'];
  
  return $multi_element;
}


function form_type_multicrud_value(&$multi_element, $edit = FALSE) {

  $multi_element['#multicrud'] += array(
    'weight_key' => 'weight',
    'parent_key' => 'parent',
    'type_key' => 'type',
  );
  
  _multicrud_expand_default_values($multi_element);
  $keymap = $multi_element['#multicrud']['keymap'];

  if (func_num_args() == 1) {
    // fresh form
    $default = $multi_element['#multicrud']['default'];
    return $default;
  } else {
    // form has POST data
    _multicrud_remove_empty_end($multi_element, $edit, $keymap);
    return $edit;
  }
}


function _multicrud_sort_items($items, $meta) {
  $children = array();
  foreach ($meta as $item_key => $item_meta) {
    $children[$item_meta['parent']][$item_key] = $item_meta['weight'];
  }
  foreach ($children as $parent_key => &$cc) {
    asort($cc);
  }
  $result = array();
  _multicrud_sort_items_rec($result, $children, $items);

  return $result;
}


function _multicrud_sort_items_rec(&$result, &$children, &$items, $key = NULL) {
  if (!empty($key)) {
    $result[$key] = $items[$key];
  }
  if (is_array($children[$key])) {
    foreach ($children[$key] as $item_key => $weight) {
      _multicrud_sort_items_rec($result, $children, $items, $item_key);
    }
  }
}


function _multicrud_expand_default_values(&$multi_element) {
  if (isset($multi_element['#multicrud_valueHandler'])) {
    $multi_element['#default_value'] = $multi_element['#multicrud_valueHandler']->getDefaultValue();
  }
  
  $default = $multi_element['#default_value'];
  $default = isset($default) ? $default : array();
  
  if (!is_array($default)) {
    $default = array('items' => array(), 'meta' => array());
  } else {
    // allow different formats for #default_value
    if (is_array($default['items'])) {
      if (!is_array($default['meta'])) {
        $default['meta'] = $default['items'];
      }
    } else {
      $default = array('items' => $default, 'meta' => $default);
    }
  }
  
  $multi_element['#multicrud']['keymap'] = $keymap = _multicrud_build_keymap(array_keys($default['items']));
  
  $item_keys = array_flip($keymap);
  $default['items'] = _multicrud_array_combine($item_keys, $default['items']);
  $meta = _multicrud_extract_meta($default['meta'], $multi_element);
  $default['meta'] = _multicrud_array_combine_special($item_keys, $meta);
  
  $default['items'] = _multicrud_sort_items($default['items'], $default['meta']);
  
  $multi_element['#multicrud']['default'] = $default;
}


function _multicrud_remove_empty_end($multi_element, &$edit, $keymap) {
  end($edit['items']);
  $last_item_key = key($edit['items']);
  if (!isset($keymap[$last_item_key])) {
    // last item is added dynamically and does not count.
    array_pop($edit['items']);
  }
}


function _multicrud_build_keymap($ids) {
  $keymap = array();
  foreach ($ids as $id) {
    $keymap["existing-$id"] = $id;
  }
  return $keymap;
}


function multicrud_validate($multi_element, &$form_state) {
  $multicrudElement = $multi_element['#multicrudElement']; // new multicrud_MulticrudElement($multi_element, $form_state['values']['form_id'], $form_state);
  $multicrudElement->setFormState($form_state);
  $itemKeys = $multicrudElement->getItemKeys();

  foreach ($itemKeys as $itemKey) {
    // get a temporary copy of $form_state
    $virtualFormState = &$form_state;
    $elements = &$multicrudElement->buildItemElements($itemKey, $virtualFormState);

    _form_validate($elements, $virtualFormState);
  }
  
  /*
  $values = $multi_element['#value'];
  $elements = $multi_element['#multicrud_elements'];
  if (!is_array($values)) {
    form_error($multi_element, t("Value should be an array."));
  }
  */
}


function _multicrud_build_element(array &$element, array $multi_element, $item_key, $element_key) {
  $form_id = NULL;
  $form_state = array();
  $element['#post'] = $multi_element['#post'];
  $element['#parents'] = array_merge($multi_element['#parents'], array('items', $item_key, $element_key));
  unset($element['#title']);
  if (!isset($element['#tree'])) {
    $element['#tree'] = TRUE;
  }
  // TODO: does this cause any conflicts or side effects?
  $element = form_builder($form_id, $element, $form_state);
}


class multicrud_MulticrudElement {
  protected $_multiElement;
  protected $_formId;
  protected $_formState;
  protected $_itemElements = array();
  protected $_default;
  
  function __construct(&$multiElement, $formId, &$formState) {
    $this->_multiElement = &$multiElement;
    $this->_default = $multiElement['#multicrud']['default'];
    $this->_formId = $formId;
    $this->_formState = &$formState;
  }
  
  function setFormState(&$formState) {
    $this->_formState = &$formState;
  }
  
  function getElementTitles() {
    $titles = array();
    $elements = $this->_multiElement['#multicrud_elements'];
    foreach (element_children($elements) as $elementKey) {
      $element = $elements[$elementKey];
      if ($element['#type'] == 'hidden') {
        // do nothing
      }
      else if ($element['#type'] == 'value') {
        // do nothing
      }
      else {
        $titles[$elementKey] = $elements[$elementKey]['#title'];
      }
    }
    return $titles;
  }
  
  function getColumns() {
    $columns = array();
    $elements = $this->_multiElement['#multicrud_elements'];
    foreach (element_children($elements) as $elementKey) {
      $element = $elements[$elementKey];
      if ($element['#type'] === 'hidden') {
        // do nothing
      }
      else if ($element['#type'] === 'value') {
        // do nothing
      }
      else {
        $columns[$elementKey]['#title'] = $elements[$elementKey]['#title'];
        $columns[$elementKey]['#description'] = $elements[$elementKey]['#description'];
      }
    }
    return $columns;
  }
  
  function getBasename() {
    return $this->_multiElement['#name'];
  }
  
  /**
   * @return TRUE, if the user should be able to sort the items.
   * (with tabledrag, if javascript is enabled)
   */
  function hasWeights() {
    return TRUE && $this->_multiElement['#multicrud_weights'];
  }
  
  /**
   * @return TRUE, if tree tabledrag sorting should be enabled.
   */
  function hasParents() {
    return !empty($this->_multiElement['#multicrud_parents']);
  }
  
  function isItem($itemKey) {
    return isset($this->_multiElement['#value']['items'][$itemKey]);
  }
  
  function isOriginalItem($itemKey) {
    return isset($this->_multiElement['#multicrud']['keymap'][$itemKey]);
    // return isset($this->_multiElement['#default_value'][$itemKey]);
  }
  
  function getItemKeys() {
    return array_keys($this->_multiElement['#value']['items']);
  }
  
  /**
   * The original key for this item, as in #default_value
   */
  function getItemId($itemKey) {
    return $this->_multiElement['#multicrud']['keymap'][$itemKey];
  }
  
  function getItemMeta($itemKey) {
    return $this->_multiElement['#value']['meta'][$itemKey];
  }
  
  /**
   * Build one row of ad-hoc elements for the given $itemKey,
   * that can be used for validation or rendering
   */
  function &buildItemElements($itemKey) {  // , &$virtualFormState = NULL) {
    if (!isset($this->_itemElements[$itemKey])) {
      $this->_itemElements[$itemKey] = $this->_buildItemElements($itemKey);
    }
    return $this->_itemElements[$itemKey];
  }
  
  
  function _buildItemElements($itemKey) {
    $virtualFormState = &$this->_formState;
    $itemId = $this->getItemId($itemKey);
    $elements = $this->_multiElement['#multicrud_elements'];
    foreach (element_children($elements) as $elementKey) {
      $element =& $elements[$elementKey];
      unset($element['#title']);
      $element['#post'] = $this->_multiElement['#post'];
      $element['#parents'] = array_merge($this->_multiElement['#parents'], array('items', $itemKey, $elementKey));
      
      if ($element['#tree'] === 'skip') {
        $element['#parents'] = array_merge($this->_multiElement['#parents'], array('items', $itemKey));
        // set #default_value
        if (isset($this->_default['items'][$itemKey])) {
          $this->_setDefaultValues($element, $this->_default['items'][$itemKey]);
        }
      }
      else {
        $element['#tree'] = TRUE;
        $element['#parents'] = array_merge($this->_multiElement['#parents'], array('items', $itemKey, $elementKey));
        // set #default_value
        if (isset($this->_default['items'][$itemKey][$elementKey])) {
          $this->_setDefaultValues($element, $this->_default['items'][$itemKey][$elementKey]);
        }
      }

      $this->_formBuilder($element, $virtualFormState);
    }
    return $elements;
  }
  
  /**
   * Set #default_value recursively for all children.
   * At this time we don't know if an element has #input,
   * so we simply set a #default_value for all elements we find.
   */
  protected function _setDefaultValues(&$element, $values) {
    $element['#default_value'] = $values;
    foreach (element_children($element) as $key) {
      if (is_array($element[$key]) && isset($values[$key])) {
        if ($element[$key]['#tree'] === 'skip') {
          $this->_setDefaultValues($element[$key], $values);
        }
        else {
          $this->_setDefaultValues($element[$key], $values[$key]);
        }
      }
    }
  }
  
  
  /**
   * Modified version of core's form_builder() function.
   * Especially, the static $complete_form is replaced with something else.
   */
  protected function _formBuilder(array &$element, array &$form_state) {
    $complete_form = $this->_multiElement['#multicrud_elements'];
    $form_id = $this->_formId;

    // Initialize as unprocessed.
    $element['#processed'] = FALSE;

    // Use element defaults.
    if ((!empty($element['#type'])) && ($info = _element_info($element['#type']))) {
      // Overlay $info onto $element, retaining preexisting keys in $element.
      $element += $info;
    }

    if (isset($element['#input']) && $element['#input']) {
      $this->_form_builder_handle_input_element($form_id, $element, $form_state, $complete_form);
    }

    $element['#defaults_loaded'] = TRUE;

    // We start off assuming all form elements are in the correct order.
    $element['#sorted'] = TRUE;

    // Recurse through all child elements.
    $count = 0;
    foreach (element_children($element) as $key) {
      $element[$key]['#post'] = $element['#post'];
      $element[$key]['#programmed'] = $element['#programmed'];
      // Don't squash an existing tree value.
      if (!isset($form[$key]['#tree'])) {
        $element[$key]['#tree'] = TRUE && $element['#tree'];
      }

      // Deny access to child elements if parent is denied.
      if (isset($element['#access']) && !$element['#access']) {
        $element[$key]['#access'] = FALSE;
      }

      // Don't squash existing parents value.
      if (!isset($element[$key]['#parents'])) {
        // Check to see if a tree of child elements is present. If so,
        // continue down the tree if required.
        if ($element[$key]['#tree'] === 'skip' && $element['#tree']) {
          $element[$key]['#parents'] = $element['#parents'];
        } else if ($element[$key]['#tree'] && $element['#tree']) {
          $element[$key]['#parents'] = array_merge($element['#parents'], array($key));
        } else {
          $element[$key]['#parents'] = array($key);
        }
        $array_parents = isset($element['#array_parents']) ? $element['#array_parents'] : array();
        $array_parents[] = $key;
        $element[$key]['#array_parents'] = $array_parents;
      }

      // Assign a decimal placeholder weight to preserve original array order.
      if (!isset($element[$key]['#weight'])) {
        $element[$key]['#weight'] = $count / 1000;
      }
      else {
        // If one of the child elements has a weight then we will need to sort
        // later.
        unset($element['#sorted']);
      }
      $this->_formBuilder($element[$key], $form_state);
      $count++;
    }

    // The #after_build flag allows any piece of a form to be altered
    // after normal input parsing has been completed.
    if (isset($element['#after_build']) && !isset($element['#after_build_done'])) {
      foreach ($element['#after_build'] as $function) {
        $element = $function($element, $form_state);
        $element['#after_build_done'] = TRUE;
      }
    }

    // Now that we've processed everything, we can go back to handle the funky
    // Internet Explorer button-click scenario.
    _form_builder_ie_cleanup($element, $form_state);

    // We shoud keep the buttons array until the IE clean up function
    // has recognized the submit button so the form has been marked
    // as submitted. If we already know which button was submitted,
    // we don't need the array.
    if (!empty($form_state['submitted'])) {
      unset($form_state['buttons']);
    }
  }


  /**
   * Copy of core's _form_builder_handle_input_element(),
   * with some additional debug statements.
   *
   * Populate the #value and #name properties of input elements so they
   * can be processed and rendered. Also, execute any #process handlers
   * attached to a specific element.
   */
  function _form_builder_handle_input_element($form_id, &$form, &$form_state, $complete_form) {
    if (!isset($form['#name'])) {
      $name = array_shift($form['#parents']);
      $form['#name'] = $name;
      if ($form['#type'] == 'file') {
        // To make it easier to handle $_FILES in file.inc, we place all
        // file fields in the 'files' array. Also, we do not support
        // nested file names.
        $form['#name'] = 'files['. $form['#name'] .']';
      }
      elseif (count($form['#parents'])) {
        $form['#name'] .= '['. implode('][', $form['#parents']) .']';
      }
      array_unshift($form['#parents'], $name);
    }
    if (!isset($form['#id'])) {
      $form['#id'] = form_clean_id('edit-'. implode('-', $form['#parents']));
    }

    if (!empty($form['#disabled'])) {
      $form['#attributes']['disabled'] = 'disabled';
    }

    if (!isset($form['#value']) && !array_key_exists('#value', $form)) {
      $function = !empty($form['#value_callback']) ? $form['#value_callback'] : 'form_type_'. $form['#type'] .'_value';
      if (($form['#programmed']) || ((!isset($form['#access']) || $form['#access']) && isset($form['#post']) && (isset($form['#post']['form_id']) && $form['#post']['form_id'] == $form_id))) {
        $edit = $form['#post'];
        foreach ($form['#parents'] as $parent) {
          $edit = isset($edit[$parent]) ? $edit[$parent] : NULL;
        }
        if (!$form['#programmed'] || isset($edit)) {
          // Call #type_value to set the form value;
          if (function_exists($function)) {
            $form['#value'] = $function($form, $edit);
          }
          if (!isset($form['#value']) && isset($edit)) {
            $form['#value'] = $edit;
          }
        }
        // Mark all posted values for validation.
        if (isset($form['#value']) || (isset($form['#required']) && $form['#required'])) {
          $form['#needs_validation'] = TRUE;
        }
      }
      // Load defaults.
      if (!isset($form['#value'])) {
        // Call #type_value without a second argument to request default_value handling.
        if (function_exists($function)) {
          $form['#value'] = $function($form);
        }
        // Final catch. If we haven't set a value yet, use the explicit default value.
        // Avoid image buttons (which come with garbage value), so we only get value
        // for the button actually clicked.
        if (!isset($form['#value']) && empty($form['#has_garbage_value'])) {
          $form['#value'] = isset($form['#default_value']) ? $form['#default_value'] : '';
        }
      }
    }

    // Determine which button (if any) was clicked to submit the form.
    // We compare the incoming values with the buttons defined in the form,
    // and flag the one that matches. We have to do some funky tricks to
    // deal with Internet Explorer's handling of single-button forms, though.
    if (!empty($form['#post']) && isset($form['#executes_submit_callback'])) {
      // First, accumulate a collection of buttons, divided into two bins:
      // those that execute full submit callbacks and those that only validate.
      $button_type = $form['#executes_submit_callback'] ? 'submit' : 'button';
      $form_state['buttons'][$button_type][] = $form;

      if (_form_button_was_clicked($form)) {
        $form_state['submitted'] = $form_state['submitted'] || $form['#executes_submit_callback'];

        // In most cases, we want to use form_set_value() to manipulate
        // the global variables. In this special case, we want to make sure that
        // the value of this element is listed in $form_variables under 'op'.
        $form_state['values'][$form['#name']] = $form['#value'];
        $form_state['clicked_button'] = $form;

        if (isset($form['#validate'])) {
          $form_state['validate_handlers'] = $form['#validate'];
        }
        if (isset($form['#submit'])) {
          $form_state['submit_handlers'] = $form['#submit'];
        }
      }
    }
    // Allow for elements to expand to multiple elements, e.g., radios,
    // checkboxes and files.
    if (isset($form['#process']) && !$form['#processed']) {
      foreach ($form['#process'] as $process) {
        if (function_exists($process)) {
          $form = $process($form, isset($edit) ? $edit : NULL, $form_state, $complete_form);
        }
      }
      $form['#processed'] = TRUE;
    }
    form_set_value($form, $form['#value'], $form_state);
  }
  
  function submit($form, &$form_state) {
    $parents = $this->_multiElement['#parents'];
    $values = $form_state['values'];
    foreach ($parents as $parent) {
      $values = $values[$parent];
    }
    multicrud_submit($this->_multiElement, $values, $this->_multiElement['#multicrud_valueHandler']);
  }
}


class multicrud_ValueHandler extends multicrud_SubmitHandler {
  function validateItem($item_values, $meta, $errorHandler) {
    // by default, nothing is validated.
  }
  
  function getDefaultValue() {
    // by default, nothing is returned.
  }
}


class multicrud_ValidationErrorHandler {
}


