


if (!Drupal.multicrud) {
  Drupal.multicrud = {};
}

if (!window.console) {
  window.console = {};
}
if (!window.console.log) {
  window.console.log = function(){};
}

$(document).ready(function(){

  var getElementValue = function(input) {
    if ($(input).is(':checkbox')) {
      return $(input).attr('checked');
    } else {
      return $(input).val();
    }
  };
  
  var setElementValue = function(input, value) {
    if ($(input).is(':checkbox')) {
      $(input).attr('checked', value ? true : false);
    } else {
      $(input).val(value);
    }
  };
  
  var buildRowKey = function(tbody) {
    for (var i=1; $('tr.new-'+i, tbody).size(); ++i) {}
    return 'new-'+i; 
  };
    
  var getRowValues = function(tr) {
    var values = {};
    $('input, textarea, select', tr).each(function(){
      var element_name = $(this).attr('name');
      values[element_name] = getElementValue(this);
    });
    return values;
  };
  
  var setRowValues = function(tr, values) {
    if ((values !== null) && typeof(values) == 'object') {
      $('input, textarea, select', tr).each(function(){
        var element_name = $(this).attr('name');
        if (values[element_name]) {
          setElementValue(this, values[element_name]);
        }
      });
    }
  };
  
  var NameBuilder = function(basename) {
    
    var items_basename = basename + '[items]';
    var meta_basename  = basename + '[meta]';
    var delete_name    = basename + '[delete]';
    
    this.buildElementName = function(old_element_name, item_key, info) {
      if (!old_element_name) {
        return null;
      }
      if (old_element_name.substring(0, basename.length) !== basename) {
        throw "old_element_name needs to begin with basename.";
      }
      var m = old_element_name.substr(basename.length).match(/^\[[\w\d_]+\]/);
      if (info) info.category = m[0];
      switch (m[0]) {
        case '[items]':
        case '[meta]':
          var old_subname = old_element_name.substr((basename+m[0]).length);
          var new_subname = old_subname.replace(/^\[[\w\d_-]+\]/, '');
          if (info) info.element_subname = new_subname;
          return basename + m[0] + '['+item_key+']' + new_subname;
        case '[delete]':
        default:
          return false;
      }
    };
  
    var _flattenNestedValues = function(prefix, nested, flat) {
      if (typeof nested == 'object') {
        for (var key in nested) {
          _flattenNestedValues(prefix + '[' + key + ']', nested[key], flat);
        }
      }
      flat[prefix] = nested;
    };
    
    this.flattenNestedValues = function(item_key, nested) {
      var flat = {};
      _flattenNestedValues(items_basename + '[' + item_key + ']', nested, flat);
      return flat;
    };
  };
    
  /**
   * Object to keep track of a row
   */
  var Row = function(tr) {
    /**
     * This function is called each time the row values are modified.
     */
    var updateRow = function(){
      var tr = this;
      
    };
  };
  
  var RowKeeper = function(empty_row, name_builder, tbody, multicrud_info) {
    
    var ref_values = getRowValues(empty_row);
    var row_keeper = this;
    var auto_add_empty_row = multicrud_info.auto_add_empty_row || !multicrud_info.explicit_add_row;
    
    var initKeeper = function() {
      $(tbody)[0].parentNode.multicrud = row_keeper;
      var rows = $('tr', tbody);
      initRow(rows[0]);
      for (var i=1; i < rows.length; ++i) {
        solidifyRow(rows[i-1]);
        initRow(rows[i]);
      }
    };
    
    var buildRow = function(row_key, values) {
      var new_row = empty_row.clone();
      var name;
      $('input, textarea, select', new_row).each(function(){
        var element_name = name_builder.buildElementName($(this).attr('name'), row_key);
        if (element_name) {
          $(this).attr('name', element_name);
        } else if ($(this).val() === 'new') {
          $(this).val(row_key);
        }
      });
      new_row
        .addClass(row_key)
        .addClass('empty')
        .removeClass('new')
      ;
      var flat_values = name_builder.flattenNestedValues(row_key, values);
      setRowValues(new_row, flat_values);
      listeners.invokeAll('buildRow', [new_row]);
      return new_row;
    };
    
    var rowIsEmpty = function(tr) {
      var is_empty = true;
      $('input, textarea, select', tr).each(function(){
        $(this).parents('td').removeClass('nonEmptyCell');
        var info = {};
        var ref_name = name_builder.buildElementName($(this).attr('name'), 'new', info);
        if (ref_name && info.category == '[items]') {
          var value = getElementValue(this);
          if (value !== null && value !== undefined && value !== ref_values[ref_name]) {
            $(this).parents('td').addClass('nonEmptyCell');
            is_empty = false;
          }
        }
      });
      return is_empty;
    };
    
    var updateRowClass = function(tr) {
      if (rowIsEmpty(tr)) {
        $(tr).addClass('empty');
      } else {
        $(tr).removeClass('empty');
      }
    };
    
    var updateRowClasses = function() {
      $('tr.item-new', tbody).each(function(){
        updateRowClass(this);
      });
    };
    
    var invokeListener = function(listener, function_name, args) {
      if (listener && listener[function_name] && typeof listener[function_name] == 'function') {
        listener[function_name].apply(listener, args);
      }
    };
    
    var listeners = new (function(){
      var _listeners = [];
      this.add = function(listener){
        _listeners.push(listener);
      };
      this.invokeAll = function(function_name, args) {
        var result = [];
        for (var i=0; i<_listeners.length; ++i) {
          result[i] = invokeListener(_listeners[i], function_name, args);
        }
        return result;
      };
    });
    
    /**
     * This function is called each time a new row is created,
     * and also on page load.
     */
    var initRow = function(tr){
      var updateRow = function(){
        row_keeper.pruneRows();
      };
      $('input, textarea, select', tr).click(updateRow).change(updateRow).keyup(updateRow);
      $(tr).each(function(){this.updateRow = updateRow;});
      listeners.invokeAll('initRow', [tr]);
    };
    
    this.addListener = function(listener) {
      $('tr', tbody).each(function(){
        invokeListener(listener, 'initRow', [this, row_keeper, tbody]);
      });
      $('tr', tbody).each(function(){
        if (!this.multicrud_nonsolid) {
          invokeListener(listener, 'solidifyRow', [this, false]);
        }
      });
      listeners.add(listener);
    };
    
    var solidifyRow = function(tr) {
      listeners.invokeAll('solidifyRow', [tr, tr.multicrud_nonsolid]);
      if (tr.multicrud_nonsolid) {
        tr.multicrud_nonsolid = false;
      }
    };
    
    var unsolidifyRow = function(tr) {
      listeners.invokeAll('unsolidifyRow', [tr, !tr.multicrud_nonsolid]);
      if (!tr.multicrud_nonsolid) {
        tr.multicrud_nonsolid = true;
      }
    };
    
    var getEmptyEnd = function() {
      var k=0, i=0;
      var rows = $('tr', tbody);
      rows.each(function(){
        ++i;
        if ($(this).is('.item-existing') || !rowIsEmpty(this)) {
          k=i;
        }
      });
      return rows.slice(k);
    };
    
    /**
     * Add a new empty row,
     * or remove some, if there are too many of them.
     */
    var pruneRowsRunning = false;
    this.pruneRows = function() {
      if (pruneRowsRunning) {
        return false;
      }
      pruneRowsRunning = true;
      if (auto_add_empty_row) {
        var rows = $('tr', tbody);
        var n = rows.size();
        var i,j;
        for (i = n-1; i >= 0; --i) {
          if ($(rows[i]).is('.item-existing')) {
            // row is for an existing item
            break;
          } else if (!rowIsEmpty(rows[i])) {
            // row is not empty
            break;
          } else if ($('input.multicrud-parent', rows[i]).val()) {
            // row has a parent
            break;
          }
          unsolidifyRow(rows[i]);
        }
        for (var j=0; j<=i; ++j) {
          solidifyRow(rows[j]);
        }
        if (n-i < 2) {
          // append one non-solid row
          var row_key = buildRowKey(tbody);
          var new_row = buildRow(row_key);
          new_row.multicrud_nonsolid = true;
          tbody.append(new_row);
          initRow(new_row);
        } else if (n-i > 2) {
          // remove some non-solid rows
          for (j = n-1; j > i+1; --j) {
            listeners.invokeAll('deleteRow', [rows[j]]);
            $(rows[j]).remove();
          }
        }
      }
      pruneRowsRunning = false;
    };
    
    this.addRowWithValues = function(values) {
      var row_key = buildRowKey(tbody);
      var new_row = buildRow(row_key, values);
      if (auto_add_empty_row) {
        $('tr:last', tbody).before(new_row);
      } else {
        $(tbody).append(new_row);
      }
      initRow(new_row);
      row_keeper.pruneRows();
    };
    
    this.getInfo = function() {
      return multicrud_info;
    };
    
    initKeeper();
  };
  
  $('tbody.multicrud-info').each(function(){
    var tbody = $(this).next('tbody');
    var table = tbody.parent();
    var multicrud_info = eval('('+$('input.multicrud-info', this).val()+')');
    $(this).remove();
    var empty_row_html = multicrud_info.empty_row_html;
    var empty_row = $(empty_row_html);
    var basename = multicrud_info.basename;
    
    var name_builder = new NameBuilder(basename);
    var row_keeper = new RowKeeper(empty_row, name_builder, tbody, multicrud_info);
    
    var update = function(){
      return;
      row_keeper.pruneRows();
      // $('a.discard-button', $('tr.item-new td.delete', tbody).slice(-1)).css('display', 'none');
      // $('a.discard-button', $('tr.item-new td.delete', tbody).slice(0, -1)).css('display', null);
    };
    
    row_keeper.pruneRows();
    
    row_keeper.addListener(new (function(){
      this.initRow = function(tr){
        $('input, textarea, select', tr)
          .click(update)
          .change(update)
          .keyup(update)
        ;
        if ($(tr).is('.item-new')) {
          var discard_button = $('<a href="" style="display:none;" class="discard-button">discard</a>');
          $('td.multicrud-delete', tr).append(discard_button);
          discard_button.click(function(event){
            $(tr).remove();
            update();
            return false;
          });
        }
      };
      this.solidifyRow = function(tr){
        $('a.discard-button', tr).css('display', '');
      };
      this.unsolidifyRow = function(tr){
        $('a.discard-button', tr).css('display', 'none');
      };
    }));
    
    tbody.each(function(){
      this.multicrud = row_keeper;
    });
    
    update();
    
    // adjust textarea heights.
    Drupal.multicrud.init.textareaMagicHeights(tbody);
    
    // delete all checkbox
    Drupal.multicrud.init.deleteAllCheckbox(tbody);
  });
});


if (!Drupal.multicrud.init) {
  Drupal.multicrud.init = {};
}


Drupal.multicrud.init.textareaMagicHeights = function(tbody) {
  var table = tbody.parent();
  var ref_textfield = $('input.form-text:first', tbody)[0];
  var ref_textarea = $('textarea', tbody)[0];
  if (ref_textfield && ref_textarea) {
    var h = ref_textfield.offsetHeight;
    $(ref_textarea).css('height', h+'px');
    h += ref_textfield.offsetHeight - ref_textarea.offsetHeight;
    $(ref_textarea).css('height', '');
    var cssRuleCode = document.all ? 'rules' : 'cssRules'; //account for IE and FF
    for (var i=0; i<document.styleSheets.length; ++i) {
      var styleSheet = document.styleSheets[i];
      if (styleSheet.href && styleSheet.href.match(/\/multicrud\.css(\?.*|#.*|)/)) {
        for (var j=0; j<styleSheet[cssRuleCode].length; ++j) {
          var rule = styleSheet[cssRuleCode][j];
          if (rule.selectorText == 'table.multicrud-table tbody textarea') {
            rule.style.height = h+'px';
          }
        }
      }
    }
  }
};


Drupal.multicrud.init.deleteAllCheckbox = function(tbody) {
  var table = tbody.parent();
  var th_delete = $('thead tr.tr-title th.multicrud-delete', table);
  var master_delete_checkbox = $('<input type="checkbox" class="master-delete-checkbox" />');
  th_delete.html('<span>&nbsp;'+th_delete.html()+'</span>');
  th_delete.prepend(master_delete_checkbox);
  
  var onchange = function(){
    var checked_now = master_delete_checkbox.attr('checked');
    $('tbody td.multicrud-delete :checkbox', table).attr('checked', checked_now);
  };
  
  master_delete_checkbox.change(onchange);
  
  // IE does not trigger the change event..
  if ($.browser.msie) {
    master_delete_checkbox.click(onchange);
  }
};




