<?php


function theme_multicrud($multi_element) {
  $multicrudElement = $multi_element['#multicrudElement'];
  $thead_html = theme('multicrud_thead', $multicrudElement);
  $tbodies_html = theme('multicrud_tbodies', $multicrudElement);
  return <<<EOT
    <div class="multicrud">
      <table class="multicrud-table">
        $thead_html
        $tbodies_html
      </table>
    </div>
EOT;
}

function theme_multicrud_thead($multicrudElement) {
  $cells = array();
  $desc_cells = array();
  $need_desc_row = FALSE;
  foreach ($multicrudElement->getColumns() as $element_key => $column) {
    $cells['td-'.$element_key] = $column['#title'];
    $desc_cells['td-'.$element_key] = $column['#description'];
    $need_desc_row = $need_desc_row || !empty($column['#description']);
  }
  if ($multicrudElement->hasWeights()) {
    $cells['multicrud-weights'] = t('Weight');
  }
  if ($multicrudElement->hasParents()) {
    $cells['multicrud-parents'] = t('Parent');
  }
  $cells['multicrud-delete'] = t('Delete');
  $tr = '';
  $tr_desc = '';
  foreach ($cells as $class => $cell_html) {
    $tr .= '<th class="'. $class .'">'. $cell_html .'</th>';
    if ($need_desc_row) {
      $tr_desc .= '<td class="'. $class .'">'. $desc_cells[$class] .'</td>';
    }
  }
  $html = '<thead class="multicrud-thead"><tr class="tr-title">'. $tr .'</tr></thead>';
  if ($need_desc_row) {
    // $html .= '<tfoot class="multicrud-tfoot"><tr class="tr-description">'. $tr_desc .'</tr></tfoot>';
  }
  return $html;
}

function theme_multicrud_tbodies($multicrudElement) {
  drupal_add_js('misc/tabledrag.js');
  drupal_add_js(drupal_get_path('module', 'multicrud') . '/multicrud.js');
  drupal_add_js(drupal_get_path('module', 'multicrud') . '/multicrud-tabledrag.js');
  drupal_add_css(drupal_get_path('module', 'multicrud') . '/multicrud.css');
  
  $rows_html = _multicrud_render_rows($multicrudElement->getItemKeys(), $multicrudElement);
  
  
  // one empty row at the bottom
  for ($i=0; $multicrudElement->isItem("new-$i"); ++$i) {}
  $itemKey = "new-$i";
  $rows_html .= _multicrud_render_rows(array($itemKey), $multicrudElement);
  
  $colspan = 1;
  // one empty row 
  $empty_row_html = _multicrud_render_rows(array("new"), $multicrudElement, $colspan);
  
  $multicrud_info_json = htmlspecialchars(json_encode(array(
    'empty_row_html' => $empty_row_html,
    'basename' => $multicrudElement->getBasename(),
    'options' => array(
      'weights' => $multicrudElement->hasWeights(),
      'parents' => $multicrudElement->hasParents(),
    ),
  )));
  
  
  return <<<EOT
    <tbody class="multicrud-info" style="display:none;">
      <tr><td colspan="$colspan">
        <input type="hidden" class="multicrud-info" value="$multicrud_info_json" />
      </td></tr>
    </tbody>
    <tbody class="multicrud-tbody">
      $rows_html
    </tbody>
EOT;
}

function _multicrud_render_rows(array $itemKeys, $multicrudElement, &$n_cells = NULL) {
  $html = '';
  foreach ($itemKeys as $item_key) {
    list($tr_cells, $tr_cells_attributes) = _multicrud_render_cells($item_key, $multicrudElement);
    $n_cells = count($tr_cells);
    $tr_html = '';
    foreach ($tr_cells as $cell_key => $cell_html) {
      $cell_attributes = $tr_cells_attributes[$cell_key];
      $cell_attributes['class'] .= ' '. check_plain($cell_key);
      $cell_attributes_html = drupal_attributes($cell_attributes);
      $tr_html .= '<td '. $cell_attributes_html .'>'. $cell_html .'</td>';
    }
    $is_new_row = !$multicrudElement->isOriginalItem($item_key);
    $row_class = ($is_new_row ? 'item-new' : 'item-existing') . ' ' .$item_key;
    $html .= <<<EOT
      <tr class="$row_class odd">$tr_html</tr>
EOT;
  }
  
  return $html;
}


function _multicrud_render_cells($item_key, $multicrudElement) {

  $elements = $multicrudElement->buildItemElements($item_key);
  $meta = $multicrudElement->getItemMeta($item_key);

  $basename = $multicrudElement->getBasename();
  $items_basename = $basename . '[items]';
  $delete_name = $basename . '[delete][]';
  
  $is_new_row = !$multicrudElement->isOriginalItem($item_key);
  
  $tr_cells = array();
  $tr_cells_attributes = array();
  $tr_hidden_html = '';
  $last_visible_element_key = NULL;
  foreach (element_children($elements) as $element_key) {
    $element = $elements[$element_key];
    $element_html = drupal_render($element);
    
    if ($element['#type'] == 'hidden') {
      $tr_hidden_html .= $element_html;
    } else if ($element['#type'] == 'value') {
      // do nothing.
    } else {
      $tr_cells['td-'.$element_key] = $element_html;
      $tr_cells_attributes['td-'.$element_key] = is_array($element['#attributes']) ? $element['#attributes'] : array();
      $last_visible_cell_key = 'td-'.$element_key;
    }
  }

  if ($multicrudElement->hasWeights()) {
    $element_name = $basename . '[meta][' . $item_key . '][weight]';
    $element_value = $meta['weight'];
    $tr_cells['multicrud-weights'] = <<<EOT
      <input class="multicrud-weight form-text" name="$element_name" value="$element_value" size="3" tabindex="111" />
EOT;
    $last_visible_cell_key = 'multicrud-weights';
  }
  if ($multicrudElement->hasParents()) {
    $element_name = $basename . '[meta][' . $item_key . '][parent]';
    $element_value = $meta['parent'];
    $tr_cells['multicrud-parents'] = <<<EOT
      <input class="multicrud-parent form-text" name="$element_name" value="$element_value" size="9" tabindex="112" />
      <input class="multicrud-id" value="$item_key" type="hidden" />
EOT;
    $last_visible_cell_key = 'multicrud-parents';
  }
  if ($is_new_row) {
    $tr_cells['multicrud-delete'] = '';
  } else {
    $tr_cells['multicrud-delete'] = <<<EOT
      <input type="checkbox" name="$delete_name" value="$item_key" tabindex="113" /> delete
EOT;
    $last_visible_cell_key = 'multicrud-delete';
  }
  if (isset($last_visible_cell_key)) {
    $tr_cells[$last_visible_cell_key] .= $tr_hidden_html;
  } else {
    $tr_cells[] = $tr_hidden_html;
  }
  return array($tr_cells, $tr_cells_attributes);
}




